# Author:: JJ Asghar <jj@chef.io>
# Cookbook Name:: chef-hipchat-plugin
# Recipe:: default
#
# Copyright 2016 Chef Software, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# What're you looking at? I know what I'm doing.
#                          - Han Solo
#

package "curl"

execute "install nodejs PPA YOLO" do
  command "curl -sL https://deb.nodesource.com/setup | sudo bash -"
  creates "/etc/apt/sources.list.d/nodesource.list"
  action :run
end

execute "Update the apt-cache" do
  command "sudo apt-get update"
  action :run
end

%w{build-essential git nodejs daemon redis-server}.each do |pkg|
  package pkg do
    action [:install]
  end
end

git "/srv/chef-hipchat-plugin" do
  repository "https://github.com/chef-partners/chef-hipchat-plugin.git"
  reference node["chef_hipchat_plugin"]["plugin_version"]
  action :checkout
end

execute "install the dependent node modules" do
  cwd "/srv/chef-hipchat-plugin"
  command "npm install"
  creates "/srv/chef-hipchat-plugin/node_modules/chef-api"
  action :run
end

cookbook_file "/etc/init.d/chef-hipchat-plugin" do
  mode "755"
  source "chef-hipchat-plugin"
end

service "redis-server" do
  supports restart: true, start: true, stop: true, reload: true
  action [:enable, :start]
end

service "chef-hipchat-plugin" do
  supports restart: true, start: true, stop: true, reload: true
  action [:enable, :start]
end
